# SourceRadar release 3 by David Bowland
# ./addons/eventscripts/sourceradar/sourceradar.py

# >>> To configure this addon please see sourceradar.cfg <<<

"""
Plays beeps for players based on the distance of opponents. Requires only ES 2.0+
"""


import es
import gamethread
import os.path


info = es.AddonInfo()
info.name = 'SourceRadar'
info.version = '3'
info.url = 'http://addons.eventscripts.com/addons/view/sourceradar'
info.basename = 'sourceradar'
info.author = 'SuperDave'


dict_players = {}
bool_radar_active = 0

list_total_sounds = ['npc/scanner/scanner_scan4.wav', 'player/geiger1.wav', 'npc/metropolice/vo/on1.wav', 'buttons/button16.wav', 'buttons/button17.wav', 'npc/roller/remote_yes.wav']
dict_options = {'sourceradar_update_delay':[2, 'Seconds between radar updates'], 'sourceradar_radar_distance':[1000, 'Distance an opponent must be within to hear a beep'], 'sourceradar_max_volume_distance':[300, 'Distance an opponent must be to hear a beep at the maximum volume'], 'sourceradar_players_on_radar':[5, 'Maximum number of beeps per radar update'], 'sourceradar_beep_delay':[0.3, 'Delay between beeps in seconds'], 'sourceradar_beep_sound':[0, 'Radar beep sound\n0 = Scanner scan4, 1 = HL2 geiger1, 2 = Metropolice on1, 3 = button16, 4 = button17, 5 = Rollermine remote_yes'], 'sourceradar_activate_radar':[0, 'Radar will be activated when there are this many players or less per team--set to 0 for always-active radar']}


def load():
   """
   Checks to make sure critical server variables are created by the config
   Creates alive players in dict_players
   Calls check_radar
   """
   global dict_players

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('sourceradar') + '/sourceradar.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/sourceradar/sourceradar.cfg')
   else:
      es.dbgmsg(0, 'SourceRadar: Unable to load sourceradar.cfg! Please ensure it is in the ./sourceradar/ directory.')

   dict_player_list = es.createplayerlist()
   for int_current_id in dict_player_list:
      if dict_player_list[int_current_id]['teamid'] > 1 and not dict_player_list[int_current_id]['isdead']:
         dict_players[str(int_current_id)] = []

   check_radar()


def es_map_start(event_var):
   """Clears the player dictionary and resets int_round_number and bool_active_radar"""
   global bool_radar_active

   dict_players.clear()
   bool_radar_active = 0
   cancel_all_delays()


def player_team(event_var):
   """Calls player_death"""
   player_death(event_var)


def player_spawn(event_var):
   """Creates a key that corresponds to a list in the player dictionary for each non-spectator player and calls check_radar"""
   global dict_players

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      del dict_players[str_userid]
   if int(event_var['es_userteam']) > 1:
      dict_players[str_userid] = []
      check_radar()


def player_death(event_var):
   """
   Deletes the player dictionary key for the dead player so the radar loops will not include the dead player
   Calls check_radar
   """
   global dict_players

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      del dict_players[str_userid]

   check_radar()


def player_disconnect(event_var):
   """Calls player_death"""
   player_death(event_var)


def unload():
   """Calls cancel_all_delays"""
   cancel_all_delays()

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def play_radar():
   """Loops through each player and plays a beep if an opponent is near"""
   global dict_players

   str_sound = list_total_sounds[int(es.ServerVar('sourceradar_beep_sound'))]
   for str_current_id in dict_players:
      if dict_players[str_current_id]:
         float_volume = dict_players[str_current_id][0]
         es.playsound(str_current_id, str_sound, float_volume)
         dict_players[str_current_id].remove(float_volume)


def update_radar():
   """
   Cancels and outstanding beep delays and clears each player's beep list
   Loops through each player and gets that player's distance from each opponent
   Creates the delays for playing beeps
   """

   dict_inner_loop_players = dict_players.copy()
   int_number_of_players = 0
   for str_current_id in dict_players:
      dict_players[str_current_id][:] = []

   for str_current_id in dict_players:
      del dict_inner_loop_players[str_current_id]
      list_current_player_location = es.getplayerlocation(str_current_id)
      int_current_player_team = es.getplayerteam(str_current_id)
      for str_inner_id in dict_inner_loop_players:
         if int_current_player_team != es.getplayerteam(str_inner_id):
            list_inner_player_location = es.getplayerlocation(str_inner_id)
            float_beep_volume = ((float(es.ServerVar('sourceradar_radar_distance')) - float(es.ServerVar('sourceradar_max_volume_distance'))) - (pow(pow(list_current_player_location[0] - list_inner_player_location[0], 2) + pow(list_current_player_location[1] - list_inner_player_location[1], 2) + pow(list_current_player_location[2] - list_inner_player_location[2], 2), 0.5) - float(es.ServerVar('sourceradar_max_volume_distance')))) / (float(es.ServerVar('sourceradar_radar_distance')) - float(es.ServerVar('sourceradar_max_volume_distance')))
            if float_beep_volume > 0:
               if float_beep_volume > 1:
                  float_beep_volume = 1
               dict_players[str_current_id].append(float_beep_volume)
               dict_players[str_inner_id].append(float_beep_volume)
      dict_players[str_current_id].sort()
      dict_players[str_current_id].reverse()

   for int_count in range(0, es.ServerVar('sourceradar_players_on_radar')):
      str_delayed_name = 'sourceradar_play_radar_%s' % int_count
      float_delay = float(es.ServerVar('sourceradar_beep_delay')) * float(int_count) + 0.1
      gamethread.delayedname(float_delay, str_delayed_name, play_radar)
   gamethread.delayedname(es.ServerVar('sourceradar_update_delay'), 'sourceradar_update_radar', update_radar)


def check_radar():
   """Checks to see if the radar should be activated or deactivated"""
   global bool_radar_active

   int_activate_radar = int(es.ServerVar('sourceradar_activate_radar'))
   if (es.getlivingplayercount(2) <= int_activate_radar and es.getlivingplayercount(3) <= int_activate_radar) or not int_activate_radar:
      if not bool_radar_active:
         bool_radar_active = 1
         gamethread.delayedname(es.ServerVar('sourceradar_update_delay'), 'sourceradar_update_radar', update_radar)
   else:
      if bool_radar_active:
         bool_radar_active = 0
         cancel_all_delays()


def cancel_all_delays():
   """Cancels all delays for beeps and radar"""
   for int_count in range(0, es.ServerVar('sourceradar_players_on_radar')):
      gamethread.cancelDelayed('sourceradar_play_radar_%s' % int_count)
   gamethread.cancelDelayed('sourceradar_update_radar')